#include <assembleit2_64.au3>

#AutoIt3Wrapper_UseX64=n


#cs _RdRand
    Use32                                                        ;32Bit!
    mov eax,1
    cpuid                                                        ;is RdRand-instruction available on your processor?
    bt ecx,30                                                    ;looking for the "yes, it is"-Bit
    mov eax,0                                                    ;exit code to AutoIt: failure
    jnc .exit                                                    ;if the Bit is not set, then exit

    ; rdrand sets CF=0 if no random number
    ; was available. Intel documentation
    ; recommends 10 retries in a tight loop
    mov ecx,11
    .loop1:
    sub ecx, 1
    jz .exit                                                     ;exit if ecx=0, 10 loops were executed
    rdrand edx                                                   ;store RdRand into Register
    jnc .loop1                                                   ;loop 10 times?

    mov edi,[esp+4]                                              ;Pointer from AutoIt structure
    _asmdbg_()                                                   ;Debugger, so you can see the whats going on :o)
    mov [edi],edx                                                ;store Register into AutoIt structure
    mov eax,1                                                    ;exit code to AutoIt: success

    .exit:
    ret

#ce


;##########first stage##########
;developer Version :o) including the ability to debug the code
$RdRandstruct = DllStructCreate("uint")
$RdRandstruct_ptr = DllStructGetPtr($RdRandstruct)

$ret = _AssembleIt2("uint", "_RdRand", "ptr", $RdRandstruct_ptr)
If $ret = 1 Then
    $RdRand = DllStructGetData($RdRandstruct, 1)
Else
    $RdRand = "Your Processor is not able to execute the Rdrand-instruction!"
EndIf
ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $RdRand = ' & $RdRand & @CRLF & '>Error code: ' & @error & @CRLF) ;### Debug Console

;##########second stage##########
;code assembled by AssembleIt
$binarycode = _AssembleIt2("retbinary", "_RdRand")               ;returns the assembled code
ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $binarycode = ' & $binarycode & @CRLF & '>Error code: ' & @error & @CRLF) ;### Debug Console


;##########third stage##########
;get the assembled code, write it into memory and call it
$binarycode = "0xB8010000000FA20FBAE11EB800000000731AB90B00000083E90174100FC7F273F68B7C24048917B801000000C3" ;returns the assembled code
$tCodeBuffer = _DllStructCreate64("byte[" & StringLen($binarycode) / 2 - 1 & "]") ;reserve Memory for opcodes
DllStructSetData($tCodeBuffer, 1, $binarycode)

$ret = DllCallAddress("uint:cdecl", DllStructGetPtr($tCodeBuffer), "ptr", $RdRandstruct_ptr) ;call the code
If $ret[0] = 1 Then
    $RdRand = DllStructGetData($RdRandstruct, 1)
Else
    $RdRand = "Your Processor is not able to execute the Rdrand-instruction!"
EndIf
ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $RdRand = ' & $RdRand & @CRLF & '>Error code: ' & @error & @CRLF) ;### Debug Console


;##########final stage##########
;make an easy function (speed is not necessary)

Func _RdRand()
    $binarycode = "0xB8010000000FA20FBAE11EB800000000731AB90B00000083E90174100FC7F273F68B7C24048917B801000000C3" ;returns the assembled code
    $tCodeBuffer = _DllStructCreate64("byte[" & StringLen($binarycode) / 2 - 1 & "]") ;reserve Memory for opcodes
    DllStructSetData($tCodeBuffer, 1, $binarycode)

    $ret = DllCallAddress("uint:cdecl", DllStructGetPtr($tCodeBuffer), "ptr", $RdRandstruct_ptr) ;call the code
    If $ret[0] = 1 Then
        $RdRand = DllStructGetData($RdRandstruct, 1)
    Else
        $RdRand = "Your Processor is not able to execute the Rdrand-instruction!"
    EndIf
    Return $RdRand
EndFunc                                                          ;==>_RdRand


;call the function
$RdRand = _RdRand()
ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $RdRand = ' & $RdRand & @CRLF & '>Error code: ' & @error & @CRLF) ;### Debug Console